(* 
   File   : /home/cvs/cvsroot/projects/visualstate/scope/BuildACFG.sml,v
   Comment: Tentative: experiments with ACFGs
   Release: scope-0-15_final 

     Copyright (C) 2003 Andrzej Wasowski
     
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.
     
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
     
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

   Besides some typical C output tools, it contains a function to generate
   eexternal functions referenced in source code. The functions generated 
   are empty return void or return true, depending on type.
   
   BuildACFG.sml,v 1.5 2003/07/08 15:45:01 wasowski Exp
*)

functor BuildACFG (S :sig type aor type atr end) :CODGEN = 
struct 

  open Types CodGen Absyn Err Cppish S

  infix 0 << <<< <<> <- (* some fun with C++ style IO *)
  infix 3 --

  structure A = ACFT 
  structure B = ACFGBuilderFn(A)
  structure V = ACFGVisFn(A)
  structure W = WuddySys

  val exists = List.exists
  val all    = List.all
  val find   = List.find

  type cmd = guard * action * stateid list (* statid always andstate *)


  datatype cnd = Event of symbol
               | Active   of symbol
	       | Inactive of symbol
  
  fun prcond (Event    e) = "Event(" ^ Symbol.name e ^ ")"
    | prcond (Active   s) = "Active(" ^ Symbol.name s ^ ")"
    | prcond (Inactive s) = "InActive(" ^ Symbol.name s ^ ")"

  (* FIXME: rudimentary *)
  fun prcmd (guard,actions,targets) :string =
	let val s =  Typeset.join_commas 
			(map Symbol.name targets) ["TGT: "]
	in String.concat (rev s) end

  type acfg = (cnd,cmd) A.acfg
  type rule = cmd B.rule
  type splitdata = (cnd,unit) B.splitdata 
	(* true if events have been matched *)

  fun takecmd (t :atr tran) :cmd = (#guard t, #action t, #targets t)

  type options = string option (* file name *)

  (* partition on top level by events (like in CodGenC1) and
     then never partition but only split by the first condition
     of the first rule on the list *)

  fun linear_bd (sys :(aor,atr) system) :acfg = 
	let val Fi     = WuddySys.build_sys sys
	    val _      = sayln Verbose (WuddySys.sprMapping ())
	    
	    fun pull_up (Fi,Lambda,e::f::es) = 
			(W.bddVar e, Event e,f::es,[])
			before sayln Verbose ("Test on Event " ^ Symbol.name e)
	      | pull_up (Fi,Lambda,_) = (* last event or none *)
			let val _ = assert (fn()=> (all (fn r => not (W.equal
						(#1 r) W.TRUE)) Lambda))
					"pull_up on matched rule"
			    val _ = assert (fn()=> (all (fn r => not (W.equal
						(#1 r) W.FALSE))Lambda))
					"pull_up on eliminated rule"
			    val r = #1 (hd Lambda)
			    val _ = assert (fn()=>not (W.equal (W.AND(Fi,r)) W.FALSE))
					"Pull_up in false branch!"
			    val _ = assert (fn()=>not (W.equal (W.AND(Fi,r)) Fi))
					"Pull_up in true branch!"
			    val sup = Vector.foldl op:: []
					(W.scanset(W.support(r)))
			    val var = valOf (List.find (not o W.is_strong Fi) sup)
					   handle e => intErr e 200301221
			    val _ = sayln Verbose ("Test on State " ^ Symbol.name var)
			    val t = fn ()=> not (SymbolEnv.defined (#evtypes sys) var)
			    val _ = assert t
				    "Events should not be in the support right now."
			in (W.bddVar var,Active var, [], []) end
			handle e => intErr e 200301224
			
				
(* TODO: I suspect that builder is wrong (probably split rules). There is 
 * some evidence that pull_up is called on blocks containg false rules 
 *)
			
	    fun partition (_,[],_) = []
	      | partition (_,Lambda,l) = [(Lambda,l)]
	      
	    val rule_bdds = map (W.build_trancond sys) (#trans sys)
	    val rule_cmds = map takecmd (#trans sys)
	    val Lambda    = ListPair.zip (rule_bdds,rule_cmds) 
	    val events    = map #1 (SE.toList (#evtypes sys))

	in B.build_acfg partition pull_up Fi Lambda events end
	handle e => intErr e 200301222

  fun print_dot (acfg :acfg) :codgen =
	cg (Dot.print (V.draw prcond prcmd acfg))

  fun compile (ofile :string option) ifile (m :(aor,atr) model) :unit = 
	let val fbase = case ofile of NONE => Path.file ifile
					    | SOME f => f
	    val os    = TextIO.openOut ( fbase ^ ".dot")
	in	
		foldl (fn (s,os) => os <- print_dot (linear_bd s)) os m;
		TextIO.closeOut os

	end handle e => intErr e 200301223




end
