(* 
   File   : /home/cvs/cvsroot/projects/visualstate/scope/CodGenC1.sml,v
   Comment: Simple code generator to ANSI C. Imitates behaviour of Interpr
	    module. This is its main module.
   Release: scope-0-15_final 

     Copyright (C) 2001-2002 Andrzej Wasowski
     
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.
     
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
     
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

   NOTES:

   This file contains the main module of CodGenC1 code generator. It performs
   the main compilation to low level representation (LR) and type inference.
   Then it calls a scheduler (CodGenC1_schedule) and dumper (CodGenC1_dump).
    
   FIXME: this only works for single system models. Switching between
   systems is not yet supported.


   1.(MINOR) signal parameters are ignored. All signals are treated as being
   without parameters (not even a warning issued for now). This is because
   current VS uses only nullary signals.  

   2. (MINOR) uninitialized variables are always initialised to value
   returned by Types.defvalty. This is not a big problem, because current VS
   will always initialise variables already in GUI (usually with zero value)

   3. (CRITICAL) no "double-buffering" yet. But it seems that strong
   assumptions from newest version of vsfsem do not demand it. (?)

   4. (COMMENT) I gave up conflict resolution. Models are simply assumed to
   be conflict free (for instance verified with no-errors). Conflict
   resolution is rather an expensive concept for a constraint resource system
   (we will evaluate it in feature). Comercial VS does not have runtime
   resolution.

   5. (COMMENT) StaticScope algorithm is used for ensuring static scopes of
   transitions. This algorithms also groups states under single scope if they
   should be enter together. 
   
   CodGenC1.sml,v 1.76 2003/07/22 08:34:33 wasowski Exp
*)

structure CodGenC1 : CODGEN = struct


(* some interface *)

open Haskellish Err CodGenC List Say Types 
infixr 1 $; 
infix 3 --;
infix 4 INIT;

(* Types *)

(* type of options for code generator *)

type aor = Absyn.attr 
type atr = Absyn.static

type options = bool * int * string option * string option * bool

(* debug, queue_len, action_prototypes_file, output file root, state-based-conf vs flags *)

type model  = (aor,atr) Absyn.model
type system = (aor,atr) Absyn.system

(* implementation *)



 
(* compile a model to LR with type resolution *)

fun comp (debug, queuelen, externhdr,ofile,confset) ifile (sys :system): unit = 
	let val lrsystem   = CodGenC1_front.comp_system (queuelen,confset) sys
				handle e => intErr e 200209261
	    val lrsizes    = CodGenC1_front.resolve_types lrsystem
				handle e => intErr e 200209262
	    val sch        = CodGenC1_schedule.schedule_system 
				confset lrsystem lrsizes
				handle e => intErr e 200209263
	    val csystem    = CodGenC1_back.cg_system (debug,externhdr,confset) 
				lrsystem lrsizes sch 
				handle e => intErr e 200209264
	in CodGenC_dump.dump_system ofile csystem end 
	handle e => intErr e 200209265 

(* main function *)  

fun compile options ifile (model :model): unit = 
	app (comp options ifile) model
	
end
