(* 
   File   : /home/cvs/cvsroot/projects/visualstate/scope/Normalise.sml,v
   Comment: Normalise Absyn model.
   Release: scope-0-15_final

     Copyright (C) 2001-2003 Andrzej Wasowski
     
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.
     
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
     
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

   NOTES:

   Normalisation currently includes: 

   1. removal of single children orstates (empty orstates are illegal at all)

   NOTE: this module is highly inefficient. This is due to extremely strong
   persistence in the hierarchy. 

   WARNING: Normalise does not work. Do not use it. Too many redundant
   states are eliminated. It might be corrected one day, but currently we
   rather know that there is nothing very normalising about this algorithm,
   so it should be moved to model optimization module. (TODO)

   WARNING: ALSO IT KNOWN THAT EVEN UNDER OLD ERORNEOUS ASSUMPTIONS THIS IS
   NOT A GOOD IMPLEMENTATION (CONTAINS A BUG TO CHASE)(TODO)
   
   Normalise.sml,v 1.22 2003/07/15 14:38:39 wasowski Exp
*)

(* some interface *)

open Absyn 
infixr 1 $;			(* Haskellish *)

type symbol = Symbol.symbol

fun reparentor newparent (ORState {name,parent,children,marking,annotation}) =
	ORState { name = name, parent = newparent, children = children,
		  marking = marking, annotation=annotation }



fun normalise (model :('aor,'atr) model) :('aor,'atr) model = 
	impossible "#200301060 Unimplemented."

(* -------------------------------------------------------------- *)
(* Eliminate group transitions                                    *)

fun simpletran (t :'atr tran) (e :event) :'atr tran =
      { trigger = Event (e,[]),
        scope = #scope t, pos = #pos t, neg = #neg t, guard = #guard t,
        action = #action t, targets = #targets t,
	annotation= #annotation t }

fun degroup (egs :evgroup env) (t :'atr tran) :'atr tran list =
      case #trigger t of
	  Event eb => [t]
	| EventGroup name => 
		let val es = Set.listItems $ SE.lookup egs name 
		in map (simpletran t) es end

fun degroup_all (egs :evgroup env) (trans :'atr tran list) :'atr tran list =
      List.concat $ map (degroup egs) trans

fun degroup_sys (sys :('aor,'atr) system) :('aor,'atr) system = 
	settrans sys (degroup_all (#evgroups sys) (#trans sys))



(* remove redundant (=ancestrally related) targets. The older of two
   can be safely removed if it does not influence the scope (for instance
   after scope computation, you can do it always). Note that this
   situation should never happen in legal models, as VisualState semantics
   does not allow ancestrally related targets. Still in mysterious way they
   appear in our sample models... *)

fun rmRedundantTgts (h :'aor hierarchy) (t :'atr tran) :'atr tran = 
       let val ts = #targets t
           fun f []       = []
             | f (he::ta) = if List.exists (ancestorh h he) ts 
			    then f ta else he::f ta
       in { trigger= #trigger t, scope = #scope t, pos = #pos t, 
            neg= #neg t, guard = #guard t, action = #action t,
            targets= f ts, annotation= #annotation t }
       end

fun rmRedundantTgts_trans (h :'aor hierarchy) (trans :'atr tran list) 
							:'atr tran list =
	map (rmRedundantTgts h) trans

fun rmRedundantTgts_sys (sys :('aor,'atr) system) :('aor,'atr) system = 
	settrans sys (rmRedundantTgts_trans (#hierarchy sys) (#trans sys))

(* move initial state to front of children list for each state *)

fun pulland denv (ANDState s) :'aor andstate = 
	ANDState { name     = #name s, 
		   parent   = #parent s,							    
		   children = map (pullor denv) (#children s),				   
		   entry    = #entry s,							   
		   exit     = #exit s }							   

and pullor denv (ORState s) :'aor orstate = 
	let val dname = SE.lookup denv (#name s)
	    val (dst,rest) = 
			List.partition (fn s => dname = nameand s) (#children s)
	in ORState { name   = #name s,
		     parent = #parent s,
		     children = map (pulland denv) (dst @ rest),
		     marking  = #marking s,
		     annotation = #annotation s }
	end

fun pull_defs (denv :denv, (root,_) :'aor hierarchy) =
	let val newroot = pulland denv root
	in (mkdenv denv newroot, idx_hierarchy newroot) end


(* alternatively: push defualt state to back of children list for each state *)
		     
fun pushand denv (ANDState s) :'aor andstate = 
	ANDState { name     = #name s, 
		   parent   = #parent s,							    
		   children = map (pushor denv) (#children s),				   
		   entry    = #entry s,							   
		   exit     = #exit s }							   

and pushor denv (ORState s) :'aor orstate = 
	let val dname = SE.lookup denv (#name s)
	    val (dst,rest) = 
			List.partition (fn s => dname = nameand s) (#children s)
	in ORState { name   = #name s,
		     parent = #parent s,
		     children = map (pushand denv) (rest @ dst),
		     marking  = #marking s,
		     annotation = #annotation s }
	end

fun push_defs (denv :denv, (root,_) :'aor hierarchy) =
	let val newroot = pushand denv root
	in (mkdenv denv newroot, idx_hierarchy newroot) end


(*
  (* reducing lvls *)

  fun reduce_lvland (s as ANDState {name,parent,children,entry,exit}) removed 
							: andstate * symbol env =
	let fun f []         done en ex removed = (done,en,ex,removed)
	      | f (os::todo) done en ex removed = 
			case childrenor os of 
				  [s] => f (todo @ map (reparentor name) (childrenand s)) 
					    done
					    (appendact (entryand s) en)
					    (appendact (exitand s) ex) 
					    (bindZip removed 
						([nameor os,nameand s],[name, name]))
					    
				| _ => let val (os1,removed1) = reduce_lvlor os removed
				       in f todo (os1 :: done) en ex removed1 end
	    val (ss1,en1,ex1,removed1) = f children [] entry exit removed
	in
		(ANDState { name= name, parent= parent, children= rev ss1, 
			      entry= en1, exit= ex1 },removed1)
	end

  and reduce_lvlor (s as ORState {name,parent,children,marking} ) remove 
						:orstate * symbol env =
	let fun f (child,(remove,children)) =
		let val (child1,remove1) = reduce_lvland child remove
		in (remove1,child1 :: children) end
	    val (remove1,children1) = foldl f (remove,[]) children
	in 
		(ORState { name=name, parent=parent, children=children1, 
			     marking= marking },remove1) 
	end

  (* second argument is the mapping of removed state names to current 
     names where they have been incorporated. It is to be used in correcting
     transitions *)

  fun norm_hierarchy (root,stenv) : hierarchy * symbol env =
	let val (root1,removed) = reduce_lvland root empty
	in if isEmpty removed then ((root,stenv),removed) 
	   else (idx_hierarchy root,removed)
	end

  (* correct transitions *)

  fun getnewor (h :hierarchy) (removed :symbol env) (old :orstate) = 
	let val _ = assert( fn () => nameor old = SE.lookup removed (nameor old) )
		(* orstate eliminated should never be referred to because
		   only scopes can be orstates and if it has been eliminated
		   it means that it was not actually a scope *)
	in valOf $ getor $ getstate h $ nameor old end

  fun getnewand (h :hierarchy) (removed :symbol env) (old :andstate)
								:andstate =
	let val name2 = SE.lookup removed (nameand old)
			handle Subscript => nameand old
	in valOf $ getand $ getstate h name2 end
	

  fun correct_tran (h :hierarchy) (removed :symbol env)  
		{trigger, scope, pos, neg, guard, action, targets} :tran =
	{ trigger = trigger,
	  scope   = getnewor h removed scope,
	  pos     = map (getnewand h removed) pos,
	  neg     = map (getnewand h removed) neg,
	  guard   = guard,
	  action  = action,
	  targets = map (getnewand h removed) targets } 

  fun correct_trans (h :hierarchy) (removed :symbol env) trans :tran list =
	map (correct_tran h removed) trans
	
  fun norm_sys (sys :system) = 
	let val (h1,removed) = norm_hierarchy (#hierarchy sys)
	    val trans1 = correct_trans h1 removed (#trans sys)
	in
		{ name = #name sys,
		  hierarchy = h1,
		  vartypes = #vartypes sys,
		  evtypes  = #evtypes sys,
		  evgroups = #evgroups sys,
		  otypes   = #otypes sys,
		  denv     = #denv sys,
		  (* should be ok to return old denv but  watch the effects ... *) 
		  venv     = #venv sys,
		  cenv     = #cenv sys,
		  trans    = trans1 }
	end

  (* main function *)  
  fun normalise (model :model) :model = map norm_sys model
*)		

