(* 
   File   : /home/cvs/cvsroot/projects/visualstate/scope/test/src/tCodGenCi.sml,v
   Comment: Test script for C1/CF code generators 
   Release: scope-0-15_final

     Copyright (C) 2001-2003 Andrzej Wasowski
     
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.
     
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
     
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
   
   NOTES:

   Main idea: translate iniconf and steps to test files which can be diffed
   with output of the program itself. steps have to be translated to temp
   input script 

   IMPORTANT: value of preopts should be defined in the point the script is used.

   tCodGenCi.sml,v 1.4 2003/07/29 14:22:00 wasowski Exp
*)
  
  app load ["Haskellish", "Cppish", "Say", "Test", "CodGen", "tInterprLang", "parse", 
            "VsrAbsyn","PP","Simplifier","Main", "MainTest"];

use "PPSymbol.sml";

open Haskellish Cppish Test tInterprLang Interpr CodGen;
        
(* fixities - Haskellish *)	    
infixr 1 $;
infix 0 << <<< <<> <-; (* some fun with C++ style IO *)
infix 3 --;

(* fixities - tInterprLang *)

infix  6 OUT;
infix  6 STATE;
infixr 5 INITIAL;
infixr 5 ||;
infixr 4 RESTART;
infixr 4 CONTINUE;
infixr 3 STEPS;
infixr 2 NEXT;

(* testcases *)

use "tInterprCases.sml";

(* history01 is known to fail on CosGenC1 as C1 has a different *)
(* semantics of history reinitialization                        *)
(* history04 fails on CodGenCF bcs of interleaving during       *)
(* exit/enters which yields different order than C1             *)

val testcases = if codgen = "C1" then 
			List.filter(fn tc=> #file tc<> "test/run/history01.vsp")
				testcases
		else cfhistory04:: 
			List.filter(fn tc=> #file tc<> "test/run/history04.vsp")
				testcases

(* where to look for scope: *)

val SCOPE = "./scope";
val testname = "fine grained CodGen" ^ codgen ^ "i " ^ preopts

val _ = Test.init testname;

val _ = system "rm -Rf test/sandbox test/sandbox2";


(* The two functions below comprise a small translator for test cases.
   The input language is the abstract syntax of tInterprLang. The output
   language is the simple test language accepted by CodGenC1_testdrv.c *)

(* TODO: only single system handled *)
(* TODO: only parameterless events can be tested *)

fun comp_in (INIT [{input=(ev,[]), endconf, outputs}]) :codgen =
      cg ["macro ", Symbol.name ev,   "\ninit\n"]
  | comp_in (CONT [{input=(ev,[]), endconf, outputs}]) :codgen =
      cg ["macro ", Symbol.name ev, "\n"]
  | comp_in _ = 
      impossible "Testing of multisystem modules or parameterized events \
	    \is not implemented.";

	
fun comp_out1 (outputs :oinst list) :codgen =
	foldl (fn (ou,e) => e -- sg (Symbol.name(#1 ou)) -- eol) eoc outputs;
	
fun comp_out iouts (INIT [{input,endconf,outputs}]) :codgen = 
			comp_out1 outputs -- iouts 
  | comp_out iouts (CONT [{input,endconf,outputs}]) :codgen =
			comp_out1 outputs
  | comp_out iouts _ = 
		impossible "Testing of multisystem modules or parameterized events \
			   \is not implemented.";


(* FIXME: single system models only are supproted *)

fun testf ({file,iniconf=[(conf,outs)],steps} : testcase) :bool =  
	let val fbase = Path.base (Path.file file)
	    val cin   = sg "init\n" -- foldr op-- eoc (map comp_in steps)
	    val iouts = comp_out1 outs
	    val cout  = iouts -- foldr op-- eoc (map (comp_out iouts) steps)
	    val _     = system "rm -Rf test/sandbox; mkdir test/sandbox"
	    val osin  = TextIO.openOut "test/sandbox/testin" 
	    val osout = TextIO.openOut "test/sandbox/testout"
	in
		osin  <- cin << "quit\n";
		osout <- cout;
		TextIO.closeOut osin;
		TextIO.closeOut osout;

		succ2bool $ system ("test/src/compile.pl " ^ file ^ " " ^ 
					codgen ^ " test/sandbox/ " ^ preopts) 
		andalso succ2bool $ system 
			("cat test/sandbox/testin | test/sandbox/system > test/sandbox/output" ) 
		andalso succ2bool $ system 
			("diff test/sandbox/testout test/sandbox/output")
	end
  | testf _ = impossible "#20020813a multi system models not supported";


val res = map (fn t => test testf t ("  " ^ #file t)) testcases; (* [List.nth(testcases,10)];  *)

val _ = Test.finalise testname res

