(* 
   File   : /home/cvs/cvsroot/projects/visualstate/scope/test/src/tRL.sml,v
   Comment: testcases for RL flattener
   Release: scope-0-15_final

     Copyright (C) 2003 Andrzej Wasowski
     
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.
     
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
     
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


   tRL.sml,v 1.4 2003/07/08 15:45:06 wasowski Exp
*)



val _ = load "tRLCommon";

structure tRL = struct

open RL

val name = "RL-flattener"

(* testcases: types *)

type input = unit frag list
type output= RL.rl option


type trestr= { vaprop : (string * vaprop) list,
                fuprop : (string * fuprop) list,
		nevere : string list }

type trl = { sys : (string * trestr) list,
	     restr : (string * trestr) list }

type testcase = string * input * (output -> bool)

(* testcases: machinery for result comparison *)

fun get_inter (rl :rl) name :restr = 
	SE.lookup (#restr rl) (S.symbol name)	(* raises NotFound *)

fun get_sys (rl :rl) name :restr =
	SE.lookup (#sys rl) (S.symbol name)	(* raises NotFound *)

fun get_fuprop (inter :restr) name :fuprop =
	SE.lookup (#fuprop inter) (S.symbol name)

fun get_vaprop (inter :restr) name :vaprop =
	SE.lookup (#vaprop inter) (S.symbol name)

fun agrees_inter (res :restr, spec :trestr) :bool = 
	Set.equal (#nevere res, Set.fromList(map S.symbol (#nevere spec)))
	andalso SE.count (#vaprop res) = length (#vaprop spec) 
	andalso SE.count (#fuprop res) = length (#fuprop spec) 
	andalso List.all (fn(n,p)=> get_fuprop res n = p) (#fuprop spec) 
	andalso List.all (fn(n,p)=> get_vaprop res n = p) (#vaprop spec) 

fun eq (spec :trl option) (res :output)= 
	(case (res,spec) of
		  (NONE,NONE) => true
		| (SOME v1, SOME v2) =>
			length (#sys v2) = SE.count (#sys v1) andalso
			length (#restr v2) = SE.count (#restr v1) andalso
			List.all (fn(n,i)=>agrees_inter 
				(get_inter v1 n,i)) 
				(#restr v2) andalso
			List.all (fn(n,i)=>agrees_inter 
				(get_sys v1  n,i))
				(#sys v2)
		| _ => false
	) handle NotFound => false

(* testcases: really now *)

val iA = {vaprop=[("a",(IntT,NONE,false))], fuprop=[], nevere=[]}
val iC = {vaprop=[("a",(IntT,NONE,false)),("b",(FloatT,NONE,false))],
	  fuprop=[], nevere=[]}
val iN = {vaprop=[], fuprop=[], nevere= ["e","e2","e1"] }
val iG = {vaprop=[], fuprop=[("g",(IntT,NONE,true))], nevere=[]}
val iH = {vaprop=[], fuprop=[("g",(IntT,NONE,true)),
			     ("h",(FloatT,NONE,true))],
	  nevere=[]}
val iJ = {vaprop=[], fuprop=[], nevere=["E1"]}
val iK = {vaprop=[], fuprop=[], nevere=["E1","E2"]}
val iL = {vaprop=[("v",(IntT,SOME (ValI 1),true))], fuprop=[],
	  nevere=[]}
val iO = {vaprop=[("v",(IntT,SOME (ValI 1),false))], fuprop=[],
	  nevere=[]}
val iP = {vaprop=[("a",(IntT,SOME (ValI 1),false))], fuprop=[],
	  nevere=[]}
val iR = {vaprop=[], fuprop=[("f",(IntT,SOME (ValI 4),true))],
	  nevere=[]}

fun CASE n snippet spec = ("snippet " ^ Int.toString n,snippet, eq spec)

val testcases :testcase list = 
	[
		CASE 0 `

			 /* simple test for neverE. */ 
			 restriction N { 
				impossible e(); 
				impossible e2(), e1(int); };

		`	 (SOME { sys = [], restr = [("N",iN)] }),

		CASE 1 `
			
			/* inheritance vars */
			restriction A { dead VS_INT a; };
			restriction C restricts A { dead VS_FLOAT b; };

		`	(SOME { sys= [], restr= [("A",iA),("C",iC)] }),
		
		CASE 2 `
		
			/* typematch */

			restriction A { dead VS_INT a; };
			restriction C restricts A { dead VS_INT a; };

		`	(SOME { sys= [], restr= [("A",iA),("C",iA)] }),

		CASE 3 `
		
			/* ERR typematch() */
			restriction A { dead VS_INT a; };
			restriction D restricts A { dead VS_FLOAT a; };

		`	NONE,

		CASE 4 `
		
			/* ERR typematch on restriction */
			restriction A { dead VS_INT a; };
			restriction E { dead VS_FLOAT a; };
			restriction F : A,E { };
			
		`	NONE,

		CASE 5 `
		
			/* inheritance functions */
			restriction G { pure VS_INT g(); };
			restriction H : G { pure VS_FLOAT h(); };

		`	(SOME { sys= [], restr= [("G",iG),("H",iH)] }),

		CASE 6 `
		
			/* ERR inheritance functions */
			restriction G { pure VS_INT g(); };
			restriction I : G { pure VS_FLOAT g(); };
		
		`	NONE,

		CASE 7 `
		
			/* inheritance nevere */
			restriction J     { impossible E1(); };
			restriction K : J { impossible E2(); };

		`	(SOME { sys=[], restr= [("J",iJ),("K",iK)] }),

		CASE 8 `
		
			/* inheritance nevere */
			restriction J     { impossible E1(); };
			restriction L : J { impossible E1(); };
		
		`	(SOME {sys=[], restr= [("J",iJ),("L",iJ)]}),

		CASE 9 `
		
			/* ERR interface duplicate */
			restriction J { impossible E1(); };
			restriction J { impossible E2(); };

		`	NONE,

		CASE 10 `
		
			/* ERR undefined interface restriction */
			restriction J : K { impossible E1(); };

		`	NONE,

		CASE 11 `
		
			/* ERR undefined interface bound to sys */
			J systemJ;
		
		`	NONE,

		CASE 12 `
		
			/* interface binding to sys */
			restriction J { impossible E1(); };
			J systemJ;

		`	(SOME { sys=[("systemJ",iJ)], restr=[("J",iJ)]}),

		CASE 13 `
		
			/* ERR rebinding system */
			restriction J  { impossible E1(); };
			restriction J1 { impossible E1(); };
			J systemJ;
			J1 systemJ;

		`	NONE,

		CASE 14 `
		 
			/* legal reconstraining */
			restriction L     { const VS_INT v = 1; };
			restriction M : L { const VS_INT v = 1; };
		
		`	(SOME {sys=[], restr=[("L",iL),("M",iL)]}),

		CASE 15 `
		
			/* snippet 15 ERR illegal reconstraining */
			restriction L     { const VS_INT v = 1; };
			restriction N : L { const VS_INT v = 2; };

		`	NONE,
		
		CASE 16 `
		
			/* constraint/dead interplay */
			restriction L { const VS_INT v = 1; };
			restriction O : L { dead VS_INT v; };

		`	(SOME {sys=[], restr=[("L",iL),("O",iO)]}),

		CASE 17 `
		
			/* snippet 17 constraint/dead interplay */
			restriction A { dead VS_INT a; };
			restriction P :A { const VS_INT a = 1; };

		`	(SOME {sys=[], restr=[("A",iA),("P",iP)]}),

		CASE 18 `
		
			/* snippet 18 constraint/pure interplay */
			restriction R { 
				const VS_INT f() = 4;
				pure VS_INT f();
			};

		`	(SOME {sys=[], restr=[("R",iR)]}),

		CASE 19 `
		
			/* snippet 19 constraint/pure interplay */
			restriction Q { 
				pure VS_INT f();
				const VS_INT f() = 4;
			};

		`	(SOME {sys=[], restr=[("Q",iR)]})

];

(* test procedure *)
	
fun run (_,snippet)  = tRLCommon.flatten snippet

end; (* structure tRL *)

(* run it! *)

structure TF = TestFn(tRL);

TF.go();

