(* 
   File   : /home/cvs/cvsroot/projects/visualstate/scope/test/src/tVsrAbsyn.sml,v
   Comment: Test script for VsrAbsyn module
   Release: scope-0-15_final

     Copyright (C) 2001-2002 Andrzej Wasowski
     
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.
     
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
     
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
     
   NOTES:

   This is very conservative set of tests. It only calls translator
   for some test modules and then calls typechecker on the results.

   If there is no uncaught exception and typechecker returns true (it mostly
   always does ha, ha...) then the test is PASSED. Otherwise a failure is
   reported.

   The test only prints the name of file tested and PASSED or *FAILED*
   message;

   Should be called as a mosml script from scope-src/ directory.

   tVsrAbsyn.sml,v 1.7 2002/12/10 19:55:57 wasowski Exp 

  *)
  
  app load ["Say","parse","VsrAbsyn","Test","Stat"];
  open Test Say;

  (* abstract syntax for specifying test cases *)
	     
           

  type testcase = { file   : string,				    
		    trans  : int option list,	
		    states : int option list,
		    depth  : int option list }

  val testcases : testcase list = 
	[{file= "test/run/actions01.vsp", trans=[SOME 1], 
		states=[SOME 4], depth=[SOME 3] },
         {file= "test/run/actions02.vsp", trans=[SOME 1], 
		states=[SOME 4], depth=[SOME 3] },
	 {file= "test/run/actions03.vsp", trans=[SOME 2],
		states=[SOME 12], depth=[SOME 7]},
	 {file= "test/run/actions04.vsp", trans=[SOME 3],
		states=[SOME 4], depth=[SOME 3]},
	 {file= "test/run/trans01.vsp", trans=[SOME 1],
		states=[SOME 3], depth=[SOME 3]},
	 {file= "test/run/trans03.vsp", trans=[SOME 1], 
		states=[SOME 5], depth=[SOME 5]},
	 {file= "test/run/trans05.vsp", trans=[SOME 1],
		states=[SOME 17], depth=[SOME 7]},
	 {file= "test/run/trans06.vsp", trans=[SOME 2],
		states=[SOME 18], depth=[SOME 7]},
	 {file= "test/run/history02.vsp", trans=[SOME 6],
		states=[SOME 8], depth=[SOME 5]},
	 {file= "test/run/peer.vsp", trans=[NONE],
		states=[NONE], depth=[NONE]},
	 {file= "test/run/Lift.vsp", trans=[SOME 19],
		states=[SOME 18], depth=[SOME 3]} ];
  
  (* fname must be a vsr or vsp filename *)
  fun testf ({file,depth,trans,states} : testcase) :bool =  
	let val parsed  = parse.parse file NONE
	    val model   = VsrAbsyn.translate parsed
	    val depth1  = map Stat.depth model
	    val trans1  = map Stat.trans model
	    val states1 = map Stat.states model
	in 
		weakly_agree depth1  depth
		andalso weakly_agree trans1  trans
		andalso weakly_agree states1 states
	end handle e => false



  val _ = say VNorm "\nConservative testing of VsrAbsyn translation:\n\n";

  val res = map (fn t => test testf t ("  " ^ #file t)) testcases;

  val _   = if List.all (fn x=>x) res 
		then say VLow "\nVsrAbsyn tests passed.\n\n"
		else say VLow "\nVsrAbsyn tests FAILED.\n\n";
		

