(* 
   File   : /home/cvs/cvsroot/projects/visualstate/scope/util/Say.sml,v
   Comment: Controlled verbosity output (AW 2002)
   Release: scope-0-15_final

     Copyright (C) 2001-2002 Andrzej Wasowski
     
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.
     
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
     
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

   Say.sml,v 1.4 2003/07/29 12:33:33 wasowski Exp
*)

structure Say :> Say = struct

        (* FIXME: currently we ignore severity level *)

	type verbosity = int

	val VOverKill: verbosity = 0  (* more than needed      *)
	val Verbose  : verbosity = 1  (* detailed debug mode   *)
	val VHigh    : verbosity = 2  (* debug mode            *)
	val VMed     : verbosity = 3  (* detailed user level   *)
	val VWarning : verbosity = 4  (* include most warnings *)
	val VNorm    : verbosity = 5  (* normal user level     *)
	val VLow     : verbosity = 6
	val VQuiet   : verbosity = 7

	fun fromInt (v :int) :verbosity = v

	val level : verbosity ref = ref VNorm

	fun setverbosity v  = level := v
	fun getverbosity () = !level
	fun printverbosity () = case !level of
		    0 => "VOverKill"
		  | 1 => "Verbose"
		  | 2 => "VHigh"
		  | 3 => "VMed"
		  | 4 => "VWarning"
		  | 5 => "VNorm"
		  | 6 => "VLow"
		  | 7 => "VQuiet"
		  | _ => "Error"

	fun say verbosity msg = 
		if verbosity >= !level then print msg
		else ();

	fun sayln verbosity msg = say verbosity (msg^"\n")

	fun println s = print s before print "\n"
	fun printi  i = print (Int.toString i)
	fun printintlist lst = app (fn i => (printi i; print ",")) lst
	fun printslist lst = app (fn s => (print s; print " ")) lst

	fun cr () = println ""

	fun printslistln lst = printslist lst before cr()

	(* conditinal operations based on verbosity *)

	fun IfVOverKill f = if !level <= VOverKill then f() else ()
	fun IfVerbose f = if !level <= Verbose then f() else ()
	fun IfVHigh   f = if !level <= VHigh   then f() else ()
	fun IfVWarning f = if !level <= VWarning then f() else ()

end
