(* 
   File   : /home/cvs/cvsroot/projects/visualstate/scope/util/Typeset.sml,v
   Comment: A set of operations on text useful in automatic formating of 
            programs. Some are vsr specific but I keep them still here.
   Release: scope-0-15_final

     Copyright (C) 2001-2002 Andrzej Wasowski
     
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.
     
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
     
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

   TODO: vsr related stuff should be moved to specific file using 
   remaining primitives (int this file). Ideally vsr code generator should be
   rewritten using CODGEN interface (so it should be called CodGenVsr).
   This file should finally be renamed to StringUtil -> a general library for
   string helpers used across projects.

   Typeset.sml,v 1.1 2002/12/11 11:56:25 wasowski Exp
*)


structure Typeset :> Typeset = struct

 (* generic indentation routines *)
 
 fun indin   tabs          = tabs ^ "\t"
 fun indent  tabs text     = tabs ^ text

 (* generic block building, separating etc *)

 fun prefix  pre      text = pre ^ text
 fun suffix      post text =       text ^ post

 fun bothfix pre post s    = concat [pre, s, post]
 fun enclose quote         = bothfix quote quote 

 fun repeat s n = let fun f 0 acc = concat acc
			| f n acc = f (n-1) (s::acc)
		  in f n [] end
 
 fun spaces n = repeat " " n
 
 fun padl n s = let val m = size s
		in if m < n then prefix (spaces (n-m)) s
			    else substring(s, 0, n)
		end
 
 val paren                 = bothfix "(" ")"
 val bracket		   = bothfix "[" "]"
 val embrace		   = bothfix " { " " } "
 val quote		   = enclose "\""
 val angle		   = bothfix "<" ">"
 
 (* compound structures builder with commas  *)
 fun vsr_block tabs title []       result = 
	tabs :: title :: ";\n\n" :: result
   | vsr_block tabs title (h::lst) result =
	let 
		val tabs' = indin tabs
		val body  = foldr (fn (s,l)=>",\n"::tabs'::s::l)
				(";\n"::result) lst 
	in  
		tabs :: title :: "\n" :: tabs' :: h :: body
	end

 (* compound structures builder without no punctuation  *)
 fun vsr_special_block tabs title []       result = 
	tabs :: title :: ";\n\n" :: result
   | vsr_special_block tabs title (h::lst) result =
	let 
		val tabs' = indin tabs
		val body  = foldr (fn (s,l)=>"\n"::tabs'::s::l)  
				(";\n"::result) lst 
	in  
		tabs :: title :: "\n" :: tabs' :: h :: body 
	end

 (*****
  * vsr block with translation	
  * transf: 'a -> string list -> string list 
  * ie: object -> accumulative param -> * acc_param_extended
  * with_representation of object
  *)

 fun vsr_block_trans title transf tabs []       result = 
	tabs :: title :: ";\n\n" :: result
   | vsr_block_trans title transf tabs (h::lst) result =
	let val tabs' = indin tabs
	in  
		tabs :: title :: "\n" :: tabs' :: transf h 
			(foldr (fn (s,r) => ",\n" :: tabs' :: transf s r) 
				(";\n\n"::result) lst)

	end

 (*****
  * as above but without inserting any punctuation 
  * (only tabs and new lines)
  *)

 fun vsr_special_block_trans title transf tabs []       result = 
	tabs :: title :: ";\n\n" :: result
   | vsr_special_block_trans title transf tabs (h::lst) result =
	let val tabs' = indin tabs
	in  
		tabs :: title :: "\n" :: tabs' :: transf h 
			(foldr (fn (s,r) => "\n" :: tabs' :: transf s r) 
				(";\n\n"::result) lst)

	end


 fun inserts sep []       result = result
   | inserts sep (h::lst) result = h :: (foldr (fn (s,l)=> sep::s::l) result lst)

 (* can be made faster by operating on lists only: 
    taking a list and producing a list *)

 fun join_commas lst result = inserts ", " lst result
 fun join_dots   lst result = inserts "."  lst result
 fun join_blanks lst result = inserts " "  lst result

 fun c_params lst result =  "(" :: (join_commas lst (")"::result))



end
